pragma solidity ^0.5.0;

import "../tokenauth/IAuthority.sol";

/**
 * @title IERCx
 *        IERCx is the interface for ERCx-compliant tokens.
 *        ERCx is defined at https://github.com/ethereum/EIPs/issues/x
 *
 * @author Jim McDonald
 */
interface IERCx {
    // Functions shared between all types of token
    function name(string calldata symbol) external view returns (string memory);
    function totalSupply(string calldata symbol) external view returns (uint256);
    function granularity(string calldata symbol) external view returns (uint256);
    function balanceOf(string calldata symbol, address owner) external view returns (uint256);

    // Functions for authority
    function establishAuthority(string calldata symbol, IAuthority authority) external;
    function getAuthorities(string calldata symbol, IAuthority authority) external view returns (IAuthority[] memory);
    function revokeAuthority(string calldata symbol, IAuthority authority) external;

    // Functions for fungible tokens
    function transfer(string calldata symbol, address holder, address recipient, uint256 amount, bytes calldata data, bytes calldata authority) external;

    // Functions for non-fungible tokens
    function create(string calldata symbol, address holder, uint256 id, bytes calldata metadata, bytes calldata authority) external;
    function send(string calldata symbol, address holder, address recipient, uint256 id, bytes calldata data, bytes calldata authority) external;

    // Functions for non-transferable tokens
    function set(string calldata symbol, address recipient, uint256 id, bytes calldata data, bytes calldata authority) external;
    function clear(string calldata symbol, address recipient, uint256 id, bytes calldata authority) external;


    // MARK AND CHEN's WEDDING 7th September

    function defaultOperators() external view returns (address[] memory);
    function authorizeOperator(address operator) external;
    function revokeOperator(address operator) external;
    function isOperatorFor(address operator, address tokenHolder) external view returns (bool);

    function send(address to, uint256 amount, bytes calldata data) external;
    function operatorSend(address from, address to, uint256 amount, bytes calldata data, bytes calldata operatorData) external;

    function burn(uint256 amount, bytes calldata data) external;
    function operatorBurn(address from, uint256 amount, bytes calldata data, bytes calldata operatorData) external;

    event Sent(address indexed operator, address indexed from, address indexed to, uint256 amount, bytes data, bytes operatorData);
    event Minted(address indexed operator, address indexed to, uint256 amount, bytes data, bytes operatorData);
    event Burned(address indexed operator, address indexed from, uint256 amount, bytes data, bytes operatorData);
    event AuthorizedOperator(address indexed operator, address indexed tokenHolder);
    event RevokedOperator(address indexed operator, address indexed tokenHolder);
}
