pragma solidity ^0.5.0;


// Copyright © 2019 Weald Technology Trading Limited
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

/**
 * @title IAuthority
 *        IAuthority is the interface for authorities.  An authority is
 *        a process that decides whether or not a transaction should proceed.
 *
 *        State of this contract: under development.
 *
 * @author Jim McDonald
 */
interface IAuthority {
    /**
     * authorise decides if the data supplied is sufficient to allow the
     * sender authority to carry out the required action.
     *
     * @param _caller the address of the contract requesting the authorisation
     * @param _sender the address that sent the transaction to the network
     * @param _holder the address of the holder of the asset
     * @param _recipient the address of the recipient of the asset
     * @param _id the identifier of the asset
     * @param _amount the amount of the asset to be transferred
     * @param _value the value sent with the transaction to the network (in Wei)
     * @param _holderData data supplied by the holder with the transaction
     * @param _callerData data supplied by the contract calling this function
     * @param _senderData data supplied by the sender with the transaction
     *
     * This will return `true` if the action is authorised, otherwise `false`
     */
    function authorise(address _caller,
                       address _sender,
                       address _holder,
                       address _recipient,
                       uint256 _id,
                       uint256 _amount,
                       uint256 _value,
                       bytes calldata _callerData,
                       bytes calldata _senderData,
                       bytes calldata _holderData)
    external returns (bool);
}
