'use strict';

const SignatureAuthority = artifacts.require('SignatureAuthority');
const AuthorityTester = artifacts.require('AuthorityTester.sol');

const truffleAssert = require('truffle-assertions');

contract('SignatureAuthority', accounts => {
    var authorityTester;
    it('initialises an authority tester', async function() {
        authorityTester = await AuthorityTester.new({
            from: accounts[0]
        });
    });

    it('only authorises if the signature is valid', async function() {
        const instance = await SignatureAuthority.new({
            from: accounts[0]
        });

        await authorityTester.addAuthority(instance.address, {
            from: accounts[1]
        });

        // Valid holder authority (sender == holder)
        await authorityTester.testAuthority(accounts[0], accounts[1], accounts[1], accounts[2], 0, 0, 0, [], [], [], {
            from: accounts[0]
        });

        // Invalid holder authority (sender != holder)
        await truffleAssert.reverts(
            authorityTester.testAuthority(accounts[0], accounts[2], accounts[1], accounts[3], 0, 0, 0, [], [], [], {
                from: accounts[0]
            }), 'no authority');

        // No authorities
        await truffleAssert.reverts(
            authorityTester.testAuthority(accounts[0], accounts[2], accounts[2], accounts[3], 0, 0, 0, [], [], [], {
                from: accounts[0]
            }), 'no authorities');
    });
});
